from .stop_words import STOP_WORDS
from .lex_attrs import LEX_ATTRS
from ...language import Language, BaseDefaults


class YorubaDefaults(BaseDefaults):
    lex_attr_getters = LEX_ATTRS
    stop_words = STOP_WORDS


class Yoruba(Language):
    lang = "yo"
    Defaults = YorubaDefaults


__all__ = ["Yoruba"]
